@@ -1,7 +1,7 @@
 /*
     Numdiff - compare putatively similar files, 
     ignoring small numeric differences
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012  Ivano Primi  <ivprimi@libero.it>
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013  Ivano Primi  <ivprimi@libero.it>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include<string.h>
 #include<ctype.h>
 #include"numdiff.h"
+#include"linesplit.h"
 
 /* See io.c */
 extern char* read_line (FILE* pf, int* errcode); 
@@ -226,7 +227,7 @@ int cmp_fields (const char* field1, const char* field2,
 static
 int cmp_lines (const char* line1, const char* line2, 
 	       unsigned long lineno1, unsigned long lineno2,  
-	       char* ifs1, char* ifs2, int output_mode, argslist* argl)
+	       const char** ifs1, const char** ifs2, int output_mode, argslist* argl)
 {
   const unsigned long fieldno_upper_limit = 8*FIELDMASK_SIZE;
 
@@ -247,6 +248,7 @@ int cmp_lines (const char* line1, const char* line2,
   else
     {
       const char *field1, *field2;
+      char *end_field1, *end_field2;
       size_t l1, l2;
       unsigned long fieldno1, fieldno2;
       Real abserr, relerr;
@@ -254,8 +256,8 @@ int cmp_lines (const char* line1, const char* line2,
 
       initR (&abserr);
       initR (&relerr);
-      field1 = line1 + strspn (line1, ifs1);
-      field2 = line2 + strspn (line2, ifs2);
+      field1 = string_spn (line1, ifs1, '\0');
+      field2 = string_spn (line2, ifs2, '\0');
       fieldno1 = fieldno2 = 0;
 
       while (*field1 != '\0' && *field2 != '\0')
@@ -266,8 +268,8 @@ int cmp_lines (const char* line1, const char* line2,
 	  while ( *field1 != '\0' && fieldno1 < fieldno_upper_limit && 
 		  (argl->ghostmask1[fieldno1 >> 3] & 0x80 >> (fieldno1 & 0x7)) )
 	    {
-	      l1 = strcspn (field1, ifs1);
-	      field1 += l1 + strspn (field1 + l1, ifs1);
+              end_field1 = string_cspn (field1, ifs1, '\0');
+	      field1 = string_spn (end_field1, ifs1, '\0');
 	      fieldno1++; 
 	    }
 	  if ( fieldno1 >= fieldno_upper_limit )
@@ -282,8 +284,8 @@ int cmp_lines (const char* line1, const char* line2,
 	  while ( *field2 != '\0' && fieldno2 < fieldno_upper_limit && 
 		  (argl->ghostmask2[fieldno2 >> 3] & 0x80 >> (fieldno2 & 0x7)) )
 	    {
-	      l2 = strcspn (field2, ifs2);
-	      field2 += l2 + strspn (field2 + l2, ifs2);
+	      end_field2 = string_cspn (field2, ifs2, '\0');
+	      field2 = string_spn (end_field2, ifs2, '\0');
 	      fieldno2++; 
 	    }
 	  if ( fieldno2 >= fieldno_upper_limit )
@@ -294,14 +296,16 @@ int cmp_lines (const char* line1, const char* line2,
 	    }
 	  if (*field1 != '\0' && *field2 != '\0')
 	    {
-	      l1 = strcspn (field1, ifs1);
-	      l2 = strcspn (field2, ifs2);
+              end_field1 = string_cspn (field1, ifs1, '\0');
+	      end_field2 = string_cspn (field2, ifs2, '\0');
+	      l1 = end_field1 - field1;
+	      l2 = end_field2 - field2;
 	      rv = cmp_fields (field1, field2, fieldno1, fieldno2, l1, l2, argl, &abserr, &relerr);
 	      if ( rv >= 1 )
 		{
 		  if ( (output_mode > OUTMODE_QUIET) && 
 		       !(rv == 1 && argl->optmask & _SE_MASK) &&
-		       !(rv == 2 && argl->optmask & _SD_MASK))
+		       !(rv == 2 && argl->optmask & _SU_MASK))
 		    {
 		      if ((_1sttime))
 			{
@@ -318,9 +322,9 @@ int cmp_lines (const char* line1, const char* line2,
 		    }
 		  lines_differ = 1;
 		}
-	      field1 += l1 + strspn (field1 + l1, ifs1);
+	      field1 = string_spn (end_field1, ifs1, '\0');
 	      fieldno1++; 
-	      field2 += l2 + strspn (field2 + l2, ifs2);
+	      field2 = string_spn (end_field2, ifs2, '\0');
 	      fieldno2++; 
 	    }
 	} /* end  while (*field1 != '\0' && *field2 != '\0') */
@@ -334,8 +338,8 @@ int cmp_lines (const char* line1, const char* line2,
       while ( *field1 != '\0' && fieldno1 < fieldno_upper_limit && 
 	      (argl->ghostmask1[fieldno1 >> 3] & 0x80 >> (fieldno1 & 0x7)) )
 	{
-	  l1 = strcspn (field1, ifs1);
-	  field1 += l1 + strspn (field1 + l1, ifs1);
+	  end_field1 = string_cspn (field1, ifs1, '\0');
+	  field1 = string_spn (end_field1, ifs1, '\0');
 	  fieldno1++; 
 	}
       if ( fieldno1 >= fieldno_upper_limit )
@@ -350,8 +354,8 @@ int cmp_lines (const char* line1, const char* line2,
       while ( *field2 != '\0' && fieldno2 < fieldno_upper_limit && 
 	      (argl->ghostmask2[fieldno2 >> 3] & 0x80 >> (fieldno2 & 0x7)) )
 	{
-	  l2 = strcspn (field2, ifs2);
-	  field2 += l2 + strspn (field2 + l2, ifs2);
+	  end_field2 = string_cspn (field2, ifs2, '\0');
+	  field2 = string_spn (end_field2, ifs2, '\0');
 	  fieldno2++; 
 	}
       if ( fieldno2 >= fieldno_upper_limit )
@@ -398,9 +402,11 @@ int cmp_lines (const char* line1, const char* line2,
     }
 }
 
+extern char** def_ifs;
+
 int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
 {
-  char *line1, *line2, *ifs1, *ifs2;
+  char *line1, *line2, **ifs1, **ifs2;
   int err1, err2, files_differ = 0;
   unsigned long lineno1 = 1, lineno2 = 1;
   size_t n;
@@ -408,8 +414,8 @@ int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
   unsigned char byte, elem;
   flg_array table = copy_of_intflagtab();
 
-  ifs1 = (!argl->ifs1) ? IFS : argl->ifs1;
-  ifs2 = (!argl->ifs2) ? IFS : argl->ifs2;
+  ifs1 = (!argl->ifs1) ? def_ifs : argl->ifs1;
+  ifs2 = (!argl->ifs2) ? def_ifs : argl->ifs2;
   if ( (table.ptr) )
     {
       /*
@@ -438,7 +444,10 @@ int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
 		case 0:
 		  goto catch_error;
 		}
-	      if ( (cmp_lines (line1, line2, lineno1, lineno2, ifs1, ifs2, argl->output_mode, argl)) )
+	      if ( (cmp_lines (line1, line2, 
+                               lineno1, lineno2, 
+                               (const char**) ifs1, (const char**) ifs2, 
+                               argl->output_mode, argl)) )
 		{
 		  files_differ = 1;
 		  if (argl->output_mode == OUTMODE_OVERVIEW)
@@ -459,7 +468,12 @@ int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
 		  lineno2++;
 		  free ((void*)line2);
 		}
-	      if (err1 == OUT_OF_MEMORY || err2 == OUT_OF_MEMORY || err1 == READING_ERROR || err2 == READING_ERROR)
+	      if (err1 == OUT_OF_MEMORY || 
+                  err2 == OUT_OF_MEMORY || 
+                  err1 == FILE_IS_BINARY || 
+                  err2 == FILE_IS_BINARY || 
+                  err1 == READING_ERROR || 
+                  err2 == READING_ERROR)
 		goto catch_error;
 	    }
 	}
@@ -473,7 +487,10 @@ int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
 	{
 	  line1 = read_line (pf1, &err1);
 	  line2 = read_line (pf2, &err2);
-	  if ( (cmp_lines (line1, line2, lineno1, lineno2, ifs1, ifs2, argl->output_mode, argl)) )
+	  if ( (cmp_lines (line1, line2, 
+                           lineno1, lineno2, 
+                           (const char**) ifs1, (const char**) ifs2, 
+                           argl->output_mode, argl)) )
 	    {
 	      files_differ = 1;
 	      if (argl->output_mode == OUTMODE_OVERVIEW)
@@ -510,8 +527,11 @@ int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
     {
       switch (err2)
 	{
+        case FILE_IS_BINARY:
+          fprintf (stderr, _("\n***  File \"%s\" is binary,\n***  cannot read from it\n"), argl->file2);
+	  return FILE_IS_BINARY;          
 	case READING_ERROR:
-	  fprintf (stderr, _("\n***  Error in reading from file \"%s\"\n"), argl->file2);
+	  fprintf (stderr, _("\n***  Error while reading from file \"%s\"\n"), argl->file2);
 	  return READING_ERROR;
 	case OUT_OF_MEMORY:
 	  fprintf (stderr, _("\n***  Out of memory while reading from file \"%s\"\n"), argl->file2);
@@ -522,9 +542,9 @@ int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
 	  /* default == EOF_REACHED */
 	default: 
 	  fprintf (stderr, _("\n***  End of file \"%s\" reached while trying to read line %lu.\n"), argl->file2, lineno2-1);
-	  fprintf (stderr, _("     File \"%s\" has more lines than file \"%s\",\n"),
+	  fprintf (stderr, _("***  File \"%s\" has more lines than file \"%s\",\n"),
 		   argl->file1, argl->file2);
-	  fprintf (stderr, _("     line %lu is the last one read from file \"%s\"\n\n"), lineno1-1, argl->file1);
+	  fprintf (stderr, _("***  line %lu is the last one read from file \"%s\"\n\n"), lineno1-1, argl->file1);
 	}
       return files_differ;
     }
@@ -532,8 +552,11 @@ int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
     {
       switch (err2)
 	{
+        case FILE_IS_BINARY:
+          fprintf (stderr, _("\n***  File \"%s\" is binary,\n***  cannot read from it\n"), argl->file2);
+	  return FILE_IS_BINARY;          
 	case READING_ERROR:
-	  fprintf (stderr, _("\n***  Error in reading from file \"%s\"\n"), argl->file2);
+	  fprintf (stderr, _("\n***  Error while reading from file \"%s\"\n"), argl->file2);
 	  return READING_ERROR;
 	case OUT_OF_MEMORY:
 	  fprintf (stderr, _("\n***  Out of memory while reading from file \"%s\"\n"), argl->file2);
@@ -542,16 +565,16 @@ int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
 	  if (getc (pf2) != EOF)
 	    {
 	      fprintf (stderr, _("\n***  End of file \"%s\" reached while trying to read line %lu.\n"), argl->file1, lineno1-1);
-	      fprintf (stderr, _("     File \"%s\" has more lines than file \"%s\",\n"),
+	      fprintf (stderr, _("***  File \"%s\" has more lines than file \"%s\",\n"),
 		       argl->file2, argl->file1);
-	      fprintf (stderr, _("     line %lu is the last one read from file \"%s\"\n\n"), lineno2-1, argl->file2);
+	      fprintf (stderr, _("***  line %lu is the last one read from file \"%s\"\n\n"), lineno2-1, argl->file2);
 	    }
 	  break;
 	case EOF_REACHED:
 	  fprintf (stderr, _("\n***  End of file \"%s\" reached while trying to read line %lu.\n"), argl->file2, lineno2-1);
-	  fprintf (stderr, _("     File \"%s\" has more lines than file \"%s\",\n"),
+	  fprintf (stderr, _("***  File \"%s\" has more lines than file \"%s\",\n"),
 		   argl->file1, argl->file2);
-	  fprintf (stderr, _("     line %lu is the last one read from file \"%s\"\n\n"), lineno1-1, argl->file1);
+	  fprintf (stderr, _("***  line %lu is the last one read from file \"%s\"\n\n"), lineno1-1, argl->file1);
 	  /*
 	    No particular action to do if err2 == LINE_INTERR
 	  */
@@ -562,8 +585,11 @@ int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
     {
       switch (err2)
 	{
+        case FILE_IS_BINARY:
+          fprintf (stderr, _("\n***  File \"%s\" is binary,\n***  cannot read from it\n"), argl->file2);
+	  return FILE_IS_BINARY;          
 	case READING_ERROR:
-	  fprintf (stderr, _("\n***  Error in reading from file \"%s\"\n"), argl->file2);
+	  fprintf (stderr, _("\n***  Error while reading from file \"%s\"\n"), argl->file2);
 	  return READING_ERROR;
 	case OUT_OF_MEMORY:
 	  fprintf (stderr, _("\n***  Out of memory while reading from file \"%s\"\n"), argl->file2);
@@ -571,15 +597,20 @@ int cmp_files (FILE* pf1, FILE* pf2, argslist* argl)
 	case OK:
 	case LINE_INTERR:
 	  fprintf (stderr, _("\n***  End of file \"%s\" reached while trying to read line %lu.\n"), argl->file1, lineno1-1);
-	  fprintf (stderr, _("     File \"%s\" has more lines than file \"%s\",\n"),
+	  fprintf (stderr, _("***  File \"%s\" has more lines than file \"%s\",\n"),
 		   argl->file2, argl->file1);
-	  fprintf (stderr, _("     line %lu is the last one read from file \"%s\"\n\n"), lineno2-1, argl->file2);
+	  fprintf (stderr, _("***  line %lu is the last one read from file \"%s\"\n\n"), lineno2-1, argl->file2);
 	}
       return files_differ;
     }
+  else if (err1 == FILE_IS_BINARY)
+    {
+      fprintf (stderr, _("\n***  File \"%s\" is binary,\n***  cannot read from it\n"), argl->file1);
+      return FILE_IS_BINARY;          
+    }
   else if (err1 == READING_ERROR)
     {
-      fprintf (stderr, _("\n***  Error in reading from file \"%s\"\n"), argl->file1);
+      fprintf (stderr, _("\n***  Error while reading from file \"%s\"\n"), argl->file1);
       return READING_ERROR;
     }
   else /* err1 == OUT_OF_MEMORY */
