@@ -37,6 +37,7 @@
 */
 
 #include "numdiff.h"
+#include "linesplit.h"
 #include <cmpbuf.h>
 #include <setmode.h>
 #include <xalloc.h>
@@ -221,7 +222,7 @@ slurp (struct file_data *current)
    class for each line.  */
 
 static void
-find_and_hash_each_line (struct file_data *current, char* ifs, int index, argslist *argl)
+find_and_hash_each_line (struct file_data *current, const char** ifs, int index, argslist *argl)
 {
   hash_value h;
   unsigned char const *p = (unsigned char const *) current->prefix_end;
@@ -256,18 +257,11 @@ find_and_hash_each_line (struct file_data *current, char* ifs, int index, argsli
       /* Hash this line until we find a newline.  */
       /* First we have to initialize `p' so that it points */
       /* to the first character of the first field */
-      for (h = 0; *p != '\n' && (strchr (ifs, *p)); p++);
+      h = 0;
+      p = (unsigned char const*)string_spn ((const char*)p, ifs, '\n');
       for (fieldno = 0; fieldno < fieldno_upper_limit && *p != '\n'; fieldno++)
         {
-          /* 
-	     The first time we enter in this cycle we have
-	     *p != '\n' && strchr (ifs, *p) == NULL
-         
-	     But (see the last line of the cycle) this
-	     is true also later 
-          */
-          for (eof = (unsigned char*)p + 1; 
-	       *eof != '\n' && !strchr (ifs, *eof); eof++);
+          eof = (unsigned char*) string_cspn ((const char*) p, ifs, '\n');
           /*
 	    We have to mark the end of the field before calling
 	    hcxnum() or acxnum().
@@ -344,8 +338,9 @@ find_and_hash_each_line (struct file_data *current, char* ifs, int index, argsli
 	    }
 
           /* Now `p' points to the end of the field */
-          for (*eof = c; *p != '\n' && (strchr (ifs, *p)); p++);
-          /* And now to the begin of the next field */
+          *eof = c;
+          p = (unsigned char const*)string_spn ((const char*)p, ifs, '\n');
+          /* And now to the beginning of the next field */
         }
       if (fieldno >= fieldno_upper_limit)
 	{
@@ -754,6 +749,8 @@ static unsigned char const prime_offset[] =
 verify (enough_prime_offsets,
 	sizeof (size_t) * CHAR_BIT <= sizeof prime_offset);
 
+extern char** def_ifs;
+
 /* Given a vector of two file_data objects, read the file associated
    with each one, and build the table of equivalence classes.
    Return nonzero if either file appears to be a binary file. */
@@ -763,10 +760,10 @@ read_files (struct file_data filevec[], argslist* argl)
 {
   int i;
   bool appears_binary = sip (&filevec[0], 0);
-  char *ifs1, *ifs2;
+  char **ifs1, **ifs2;
 
-  ifs1 = (!argl->ifs1) ? IFS : argl->ifs1;
-  ifs2 = (!argl->ifs2) ? IFS : argl->ifs2;
+  ifs1 = (!argl->ifs1) ? def_ifs : argl->ifs1;
+  ifs2 = (!argl->ifs2) ? def_ifs : argl->ifs2;
   if (filevec[0].desc != filevec[1].desc)
     appears_binary |= sip (&filevec[1], appears_binary); /*# skip test if appears_binary is already !0 #*/
 /*   else */
@@ -799,8 +796,8 @@ read_files (struct file_data filevec[], argslist* argl)
   buckets = zalloc ((nbuckets + 1) * sizeof *buckets);
   buckets++;
 
-  find_and_hash_each_line (&filevec[0], ifs1, 0, argl);
-  find_and_hash_each_line (&filevec[1], ifs2, 1, argl);
+  find_and_hash_each_line (&filevec[0], (const char**) ifs1, 0, argl);
+  find_and_hash_each_line (&filevec[1], (const char**) ifs2, 1, argl);
 
   filevec[0].equiv_max = filevec[1].equiv_max = equivs_index;
 
