@@ -1,7 +1,7 @@
 /*
     Numdiff - compare putatively similar files, 
     ignoring small numeric differences
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012  Ivano Primi  <ivprimi@libero.it>
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013  Ivano Primi  <ivprimi@libero.it>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,8 +37,11 @@ typedef struct {
   /* First field, last field, increment */
   unsigned long first_field, last_field, increment;
 
-  /* Internal fields separator */
-  char *ifs;
+  /* Internal fields separators */
+  char **ifs;
+
+  /* Output separator */
+  char *osep;
 
   /* File to read from */
   const char *file;
@@ -53,8 +56,10 @@ typedef struct {
 #define ___SL_MASK  0x00000020 /* -L option, used to set the last field to display */
 #define ___SI_MASK  0x00000040 /* -I option, used to explicitly set the increment */
 #define ___SS_MASK  0x00000080 /* -S option, used to explicitly set IFS */
-#define ___X_MASK   0x00000100 /* -x option, to omit empty lines */
-#define ___V_MASK   0x00000200 /* -v option, used to show version number,
+#define ___SD_MASK  0x00000100 /* -D option, used to explicitly set IFS */
+#define ___SO_MASK  0x00000200 /* -O option, used to set the output separator */
+#define ___X_MASK   0x00000400 /* -x option, to omit empty lines */
+#define ___V_MASK   0x00000800 /* -v option, used to show version number,
 				  Copyright and No-Warranty */
 
 /* I18N and L10N support */
