@@ -1,7 +1,7 @@
 /*
     Numdiff - compare putatively similar files, 
     ignoring small numeric differences
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012  Ivano Primi  <ivprimi@libero.it>
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013  Ivano Primi  <ivprimi@libero.it>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,14 +41,15 @@ verify (hash_value_is_unsigned, ! TYPE_SIGNED (hash_value));
 #endif
 
 /* Error codes */
-#define OK            0
-#define LINE_INTERR   1
-#define EOF_REACHED   2
-#define READING_ERROR 3
-#define OUT_OF_MEMORY 4
+#define OK             0
+#define LINE_INTERR    1
+#define EOF_REACHED    2
+#define READING_ERROR  3
+#define OUT_OF_MEMORY  4
 /* *** */
-#define OPEN_FAILED   5
-#define WRONG_USAGE   6
+#define OPEN_FAILED    5
+#define WRONG_USAGE    6
+#define FILE_IS_BINARY 7
 
 /*
   Begin section -- Math types
@@ -215,7 +216,7 @@ typedef struct {
   const char *file1, *file2;
 
   /* Internal fields separators (IFS) for file1 and file2 */
-  char *ifs1, *ifs2;
+  char **ifs1, **ifs2;
 
   /* Numeric conventions for file1 (.nf1) and file2 (.nf2) */
   struct numfmt nf1, nf2;
@@ -255,7 +256,7 @@ typedef struct {
 		  	    	lines of the files must be ignored */
 #define _SP_MASK  0x00200000 /* -P option, used to ignore negative errors */
 #define _SN_MASK  0x00400000 /* -N option, used to ignore positive errors */
-#define _SD_MASK  0x00800000 /* -D option, used to enable the "dummy" mode */
+#define _SU_MASK  0x00800000 /* -U option, used to enable the "dummy" mode */
 #define _SE_MASK  0x01000000 /* -E option, used to enable the "essential" 
 				mode */
 #define _SV_MASK  0x02000000 /* -V option, used to enable the "verbose" mode */
@@ -326,7 +327,6 @@ typedef struct {
   .nf*.ech      (prefix for decimal exponent)
   .nf*.iu       (symbol of the imaginary unit)
   .iscale      (decimal digits of accuracy)
-  .ifs         (internal fields separator)
 */
 #define CURRENCY  ""
 #define DP        '.'
@@ -337,7 +337,6 @@ typedef struct {
 #define ECH       'e'
 #define IU        'i'
 #define ISCALE    35
-#define IFS  " \t\n"
 
 
 /*
