@@ -1,7 +1,7 @@
 /*
     Numdiff - compare putatively similar files, 
     ignoring small numeric differences
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012  Ivano Primi  <ivprimi@libero.it>
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013  Ivano Primi  <ivprimi@libero.it>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include<float.h>
 #include"getopt.h"
 #include"error.h"
+#include"linesplit.h"
 #include"xalloc.h"
 
 #ifdef _DMALLOC_
@@ -38,7 +39,7 @@
 void print_version (const char* progname)
 {
   printf ("%s %s\n", progname, VERSION);
-  printf ("Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012  %s <ivprimi@libero.it>\n", 
+  printf ("Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013  %s <ivprimi@libero.it>\n", 
 	  /* TRANSLATORS: This is a proper name.  See the gettext
 	     manual, section Names.
 	     Pronounciation is like "evaa-no pree-me".  */
@@ -62,7 +63,7 @@ void print_help (const char* progname)
 {
   puts (_("Usage:"));
   printf ("%s -h|--help|-v|--version   %s\n\n", progname, _("or"));
-  printf ("%s %s\n", progname, "[-s IFS][-a THRVAL[:RANGE|:RANGE1:RANGE2]][-r THRVAL[:RANGE|:RANGE1:RANGE2]][-2][-F NUM][-# NUM][-P][-N][-I][-c CURRNAME][-d C1C2][-t C1C2][-g N1N2][-p C1C2][-n C1C2][-e C1C2][-i C1C2][-X 1:RANGE][-X 2:RANGE][-E][-D][-b][-V][-O[NUM]][-q][-S][-z 1:RANGE][-z 2:RANGE][-Z 1:RANGE][-Z 2:RANGE][-m][-H][-f[NUM]][-T][-B][-l PATH][-o PATH] FILE1 FILE2");
+  printf ("%s %s\n", progname, "[-s IFS][-D DELIMS][-a THRVAL[:RANGE|:RANGE1:RANGE2]][-r THRVAL[:RANGE|:RANGE1:RANGE2]][-2][-F NUM][-# NUM][-P][-N][-I][-c CURRNAME][-d C1C2][-t C1C2][-g N1N2][-p C1C2][-n C1C2][-e C1C2][-i C1C2][-X 1:RANGE][-X 2:RANGE][-E][-U][-b][-V][-O[NUM]][-q][-S][-z 1:RANGE][-z 2:RANGE][-Z 1:RANGE][-Z 2:RANGE][-m][-H][-f[NUM]][-T][-B][-l PATH][-o PATH] FILE1 FILE2");
   printf (_("\nCompare putatively similar files line by line and field by field,\nignoring small numeric differences or/and different numeric formats.\n\n"));
   printf (_("RANGE, RANGE1 and RANGE2 stay for a positive integer value or\nfor a range of integer values, like 1-, 3-5 or -7.\n"));
   printf ("%s\n%s\n%s\n\n%s\n\n",
@@ -71,10 +72,14 @@ void print_help (const char* progname)
 	  _("The given paths cannot refer to the same file\nbut one of them can be \"-\", which refers to stdin."),
 	  _("Exit status: 1 if files differ, 0 if they are equal, -1 (255) in case of error"));
   /* %%% */
-  printf ("-s, --separator=IFS\n    %s\n    %s\n    %s\n",
-	  _("Specify the set of characters to use\n    to split the input lines into fields"),
-	  _("(The default set of characters is space, tab and newline)."),
-	  _("If IFS is prefixed with 1: or 2: then use the given character set\n    only for the lines from the first or the second file respectively"));
+  printf ("-s, --separators=IFS\n    %s\n    %s\n    %s\n",
+	  _("Specify the set of characters to use as delimiters\n    while splitting the input lines into fields"),
+	  _("(The default set of delimiters is space, tab and newline)."),
+	  _("If IFS is prefixed with 1: or 2: then use the given delimiter set\n    only for the lines from the first or the second file respectively"));
+  printf ("-D, --delimiters=DELIMS\n    %s\n    %s\n    %s\n",
+	  _("Specify the set of strings to use as delimiters\n    while splitting the input lines into fields"),
+	  _("(The default set of delimiters is space, tab and newline)."),
+	  _("If DELIMS is prefixed with 1: or 2: then use the given delimiter set\n    only for the lines from the first or the second file respectively"));
   printf ("-a, --absolute-tolerance=THRVAL[:RANGE|:RANGE1:RANGE2]\n    %s\n    %s\n    %s\n", 
 	  _("Set to THRVAL the maximum absolute difference permitted\n    before that two numeric fields are regarded as different\n    (The default value is zero)."),
 	  _("If a RANGE is given, use the specified\n    threshold only when comparing fields whose positions lie in RANGE."),
@@ -121,7 +126,7 @@ void print_help (const char* progname)
 	  _("Select the fields of the second file that have to be ignored"));
   printf ("-E, --essential\n    %s\n",
 	  _("While printing the differences between the two compared files\n    show only the numerical ones"));
-  printf ("-D, --dummy\n    %s\n",
+  printf ("-U, --dummy\n    %s\n",
 	  _("While printing the differences between the two compared files\n    neglect all the numerical ones (dummy mode)"));
   printf ("-b, --brief\n    %s\n",
 	  _("Suppress all messages concerning the differences discovered\n    in the structures of the two files"));
@@ -177,49 +182,6 @@ void print_help (const char* progname)
   printf (_("Symbol used to denote the imaginary unit = `%c\'\n\n"), IU);
 }
 
-static
-char* return_ifs (const char* optarg)
-{
-  char *s = (char*) malloc ((strlen(optarg) + 1) * sizeof(char));
-  
-  if(!s)
-    return NULL;
-  else
-    {
-      char *t, *u;
-
-      strcpy (s, optarg);
-      for (t = s; *t != '\0'; t++)
-	{
-	  if (*t == '\\')
-	    {
-	      switch (*(t+1))
-		{
-		case 'f':
-		  *t = '\f';
-		  break;
-		case 'n':
-		  *t = '\n';
-		  break;
-		case 'r':
-		  *t = '\r';
-		  break;
-		case 't':
-		  *t = '\t';
-		  break;
-		case 'v':
-		  *t = '\v';
-		  break;
-		default:
-		  *t = *(t+1);
-		}
-	      for (u = t+1; *u != '\0'; *u = *(u+1), u++);
-	    }
-	}
-      return s;
-    }
-}
-
 static
 int nfset (int opt_ch, const char* opt_arg, argslist* arg_list)
 {
@@ -392,7 +354,7 @@ extern int optind;
 int setargs (int argc, char* argv[], argslist *list)
 {
   const int mask_size = FIELDMASK_SIZE*8;
-  const char *optstring = "h2F:bBVO::qDESIPNz:Z:mHT#:s:a:r:c:d:t:g:p:n:e:i:f::X:l:o:v";
+  const char *optstring = "h2F:bBVO::qUESIPNz:Z:mHT#:s:D:a:r:c:d:t:g:p:n:e:i:f::X:l:o:v";
   struct option long_options[] = {
     {"help",                 0, NULL, 'h'},
     {"strict",               0, NULL, '2'},
@@ -403,7 +365,7 @@ int setargs (int argc, char* argv[], argslist *list)
     {"overview",             2, NULL, 'O'},
     {"quiet",                0, NULL, 'q'},
     {"silent",               0, NULL, 'q'},
-    {"dummy",                0, NULL, 'D'},
+    {"dummy",                0, NULL, 'U'},
     {"essential",            0, NULL, 'E'},
     {"statistics",           0, NULL, 'S'},
     {"ignore-case",          0, NULL, 'I'},
@@ -415,7 +377,8 @@ int setargs (int argc, char* argv[], argslist *list)
     {"speed-large-files",    0, NULL, 'H'},
     {"expand-tabs",          0, NULL, 'T'},
     {"digits",               1, NULL, '#'},
-    {"separator",            1, NULL, 's'},
+    {"separators",           1, NULL, 's'},
+    {"delimiters",           1, NULL, 'D'},
     {"absolute-tolerance",   1, NULL, 'a'},
     {"relative-tolerance",   1, NULL, 'r'},
     {"currency",             1, NULL, 'c'},
@@ -479,8 +442,8 @@ int setargs (int argc, char* argv[], argslist *list)
   list->nf1.iu = IU;
   list->file1 = list->file2 = NULL;
   list->nf2 = list->nf1;
-  list->nf1.currency = xstrdup (CURRENCY);
-  list->nf2.currency = xstrdup (CURRENCY);
+  list->nf1.currency = get_separating_string (CURRENCY);
+  list->nf2.currency = get_separating_string (CURRENCY);
 
   while ( (optch = getopt_long (argc, argv, optstring, long_options, &option_index)) != -1 )
     {
@@ -518,8 +481,8 @@ int setargs (int argc, char* argv[], argslist *list)
 	case 'q':
 	  list->optmask |= _Q_MASK;
 	  break;
-	case 'D':
-	  list->optmask |= _SD_MASK;
+	case 'U':
+	  list->optmask |= _SU_MASK;
 	  break;
 	case 'E':
 	  list->optmask |= _SE_MASK;
@@ -626,48 +589,136 @@ int setargs (int argc, char* argv[], argslist *list)
 	case 's':
 	  if (*optarg == '1' && *(optarg+1) == ':')
 	    {
-	      if ((list->ifs1))
-		{
-		  free((void*)list->ifs1);
-		  list->ifs1 = NULL;
-		}
-	      list->ifs1 = return_ifs (optarg+2);
+              if ((list->ifs1))
+                {
+                  delete_string_vector (list->ifs1);
+                  list->ifs1 = NULL;
+                }
+              list->ifs1 = ssplit_former_way (optarg+2);
+              if (!list->ifs1)
+                {
+                  fprintf (stderr, _("%s: memory exhausted\n"), PACKAGE);
+                  return -1;
+                }
 	    }
 	  else if (*optarg == '2' && *(optarg+1) == ':')
 	    {
-	      if ((list->ifs2))
-		{
-		  free((void*)list->ifs2);
-		  list->ifs2 = NULL;
-		}
-	      list->ifs2 = return_ifs (optarg+2);
+              if ((list->ifs2))
+                {
+                  delete_string_vector (list->ifs2);
+                  list->ifs2 = NULL;
+                }
+              list->ifs2 = ssplit_former_way (optarg+2);
+              if (!list->ifs2)
+                {
+                  fprintf (stderr, _("%s: memory exhausted\n"), PACKAGE);
+                  return -1;
+                }
 	    }
 	  else
 	    {
-	      if ((list->ifs1))
-		{
-		  free((void*)list->ifs1);
-		  list->ifs1 = NULL;
-		}
-	      if ((list->ifs2))
-		{
-		  free((void*)list->ifs2);
-		  list->ifs2 = NULL;
-		}
-	      list->ifs1 = return_ifs (optarg);
-	      list->ifs2 = return_ifs (optarg);
+              if ((list->ifs1))
+                {
+                  delete_string_vector (list->ifs1);
+                  list->ifs1 = NULL;
+                }
+              if ((list->ifs2))
+                {
+                  delete_string_vector (list->ifs2);
+                  list->ifs2 = NULL;
+                }
+              list->ifs1 = ssplit_former_way (optarg);
+              list->ifs2 = ssplit_former_way (optarg);
+              if (!list->ifs1 || !list->ifs2)
+                {
+                  fprintf (stderr, _("%s: memory exhausted\n"), PACKAGE);
+                  return -1;
+                }
 	    }
-	  if ( ((list->ifs1) && !strchr(list->ifs1, NEWLINE)) ||
-	       ((list->ifs2) && !strchr(list->ifs2, NEWLINE)) )
+
+          if ( ((list->ifs1) && !is_string_in_vector(NEWLINE_STR, (const char**) list->ifs1)) ||
+               ((list->ifs2) && !is_string_in_vector(NEWLINE_STR, (const char**) list->ifs2)) )
 	    {
 	      fprintf (stderr, _("%s: invalid argument after `-%c\' option:\n"),
 		       PACKAGE, optch);
-	      fprintf (stderr, _("  A list of field delimiters can not be empty and\n  must always include the newline character (\'\\n\')\n"));
+	      fprintf (stderr, _("  The list of field delimiters can not be empty and\n  must always include the newline character (\'\\n\')\n"));
 	      return -1;
 	    }
 	  else
-	    list->optmask |= _S_MASK;
+            {
+              remove_duplicates_from_string_vector (list->ifs1);
+              remove_duplicates_from_string_vector (list->ifs2);
+              sort_string_vector (list->ifs1); /* This is not strictly necessary */
+              sort_string_vector (list->ifs2); /* This is not strictly necessary */              
+              list->optmask |= _S_MASK;
+            }
 	  break;
+        case 'D':
+	  if (*optarg == '1' && *(optarg+1) == ':')
+	    {
+              if ((list->ifs1))
+                {
+                  delete_string_vector (list->ifs1);
+                  list->ifs1 = NULL;
+                }
+              list->ifs1 = ssplit (optarg+2, I_DEF_SEP);
+              if (!list->ifs1)
+                {
+                  fprintf (stderr, _("%s: memory exhausted\n"), PACKAGE);
+                  return -1;
+                }
+	    }
+	  else if (*optarg == '2' && *(optarg+1) == ':')
+	    {
+              if ((list->ifs2))
+                {
+                  delete_string_vector (list->ifs2);
+                  list->ifs2 = NULL;
+                }
+              list->ifs2 = ssplit (optarg+2, I_DEF_SEP);
+              if (!list->ifs2)
+                {
+                  fprintf (stderr, _("%s: memory exhausted\n"), PACKAGE);
+                  return -1;
+                }
+	    }
+	  else
+	    {
+              if ((list->ifs1))
+                {
+                  delete_string_vector (list->ifs1);
+                  list->ifs1 = NULL;
+                }
+              if ((list->ifs2))
+                {
+                  delete_string_vector (list->ifs2);
+                  list->ifs2 = NULL;
+                }
+              list->ifs1 = ssplit (optarg, I_DEF_SEP);
+              list->ifs2 = ssplit (optarg, I_DEF_SEP);
+              if (!list->ifs1 || !list->ifs2)
+                {
+                  fprintf (stderr, _("%s: memory exhausted\n"), PACKAGE);
+                  return -1;
+                }
+	    }
+          if ( ((list->ifs1) && is_char_in_vector(NEWLINE, (const char**) list->ifs1) != 1) ||
+               ((list->ifs2) && is_char_in_vector(NEWLINE, (const char**) list->ifs2) != 1) )
+	    {
+	      fprintf (stderr, _("%s: invalid argument after `-%c\' option:\n"),
+		       PACKAGE, optch);
+	      fprintf (stderr, _("  The list of field delimiters cannot be empty and\n  must always include the newline string (\"\\n\").\n  Care that the newline character cannot appear\n  in any other delimiter than the newline string\n"));
+	      return -1;
+	    }
+	  else
+            {
+              remove_duplicates_from_string_vector (list->ifs1);
+              remove_duplicates_from_string_vector (list->ifs2);
+              sort_string_vector (list->ifs1); 
+              sort_string_vector (list->ifs2); 
+              list->optmask |= _S_MASK;
+            }
+          break;
 	case 'a':
 	  rv = thrlist_add (&list->maxabserr, optarg);
 	  if (rv == THRLIST_INVALID_FORMAT)
@@ -712,24 +763,24 @@ int setargs (int argc, char* argv[], argslist *list)
 	    {
 	      if ((list->nf1.currency))
 		free((void*)list->nf1.currency);
-	      list->nf1.currency = xstrdup (optarg+2);
+	      list->nf1.currency = get_separating_string (optarg+2);
               file_id = 0x1;
 	    }
 	  else if (*optarg == '2' && *(optarg+1) == ':')
 	    {
 	      if ((list->nf2.currency))
 		free((void*)list->nf2.currency);
-	      list->nf2.currency = xstrdup (optarg+2);
+	      list->nf2.currency = get_separating_string (optarg+2);
 	      file_id = 0x2;
 	    }
 	  else
 	    {
 	      if ((list->nf1.currency))
 		free((void*)list->nf1.currency);
-	      list->nf1.currency = xstrdup (optarg);
+	      list->nf1.currency = get_separating_string (optarg);
 	      if ((list->nf2.currency))
 		free((void*)list->nf2.currency);
-	      list->nf2.currency = xstrdup (optarg);
+	      list->nf2.currency = get_separating_string (optarg);
 	      file_id = 0x3;
 	    }
 	  if ( (strlen(list->nf1.currency) == 0 && ((file_id & 0x3) == 0x1))
