@@ -37,6 +37,7 @@
 */
 
 #include "numdiff.h"
+#include "linesplit.h"
 #include <error.h>
 #include <xalloc.h>
 
@@ -70,6 +71,8 @@ pfatal_with_name (char const *name)
  * from the same file.
  */
 
+extern char** def_ifs;
+
 bool
 lines_differ (char const *s1, char const *s2, int index1, int index2, argslist* argl)
 {
@@ -82,22 +85,22 @@ lines_differ (char const *s1, char const *s2, int index1, int index2, argslist*
 
   /* Cache often-used quantities in local variables to help the compiler.  */
   int ignore_case = argl->optmask & _SI_MASK;
-  char* ifs = (index1) ? argl->ifs2 : argl->ifs1; 
+  const char** ifs = (const char**)((index1) ? argl->ifs2 : argl->ifs1); 
   const struct numfmt* pnf = (index1) ? &argl->nf2 : &argl->nf1; 
   unsigned char* ghostmask = (index1) ? argl->ghostmask2 : argl->ghostmask1;
   unsigned char* pblurmask = (index1) ? argl->pblurmask2 : argl->pblurmask1;
   unsigned char* tblurmask = (index1) ? argl->tblurmask2 : argl->tblurmask1; /* s1 */
 
-  char* Ifs = (index2) ? argl->ifs2 : argl->ifs1; 
+  const char** Ifs = (const char**)((index2) ? argl->ifs2 : argl->ifs1); 
   const struct numfmt* Pnf = (index2) ? &argl->nf2 : &argl->nf1; 
   unsigned char* Ghostmask = (index2) ? argl->ghostmask2 : argl->ghostmask1;
   unsigned char* Pblurmask = (index2) ? argl->pblurmask2 : argl->pblurmask1;
   unsigned char* Tblurmask = (index2) ? argl->tblurmask2 : argl->tblurmask1; /* s2 */
 
-  ifs = (!ifs) ? IFS : ifs;
-  Ifs = (!Ifs) ? IFS : Ifs;
-  for (; *f1 != '\n' && (strchr (ifs, *f1)); f1++);
-  for (; *f2 != '\n' && (strchr (Ifs, *f2)); f2++);
+  ifs = (!ifs) ? (const char**)def_ifs : ifs;
+  Ifs = (!Ifs) ? (const char**)def_ifs : Ifs;
+  f1 = string_spn (f1, ifs, '\n');
+  f2 = string_spn (f2, Ifs, '\n');
   fieldno1 = fieldno2 = 0;
 
   while (*f1 != '\n' && *f2 != '\n')
@@ -108,8 +111,8 @@ lines_differ (char const *s1, char const *s2, int index1, int index2, argslist*
       while ( *f1 != '\n' && fieldno1 < fieldno_upper_limit && (ghostmask[fieldno1 >> 3] & 0x80 >> (fieldno1 & 0x7)) )
 	{
 	  /* First move `f1' to the begin of the next field */
-	  for (; *f1 != '\n' && !strchr (ifs, *f1); f1++);
-	  for (; *f1 != '\n' && (strchr (ifs, *f1)); f1++);
+          e1 = string_cspn (f1, ifs, '\n');
+          f1 = string_spn (e1, ifs, '\n');
 	  /* and then increment the field index */
 	  fieldno1++; 
 	}
@@ -123,8 +126,8 @@ lines_differ (char const *s1, char const *s2, int index1, int index2, argslist*
       while ( *f2 != '\n' && fieldno2 < fieldno_upper_limit && (Ghostmask[fieldno2 >> 3] & 0x80 >> (fieldno2 & 0x7)) )
 	{
 	  /* First move `f2' to the begin of the next field */
-	  for (; *f2 != '\n' && !strchr (Ifs, *f2); f2++);
-	  for (; *f2 != '\n' && (strchr (Ifs, *f2)); f2++);
+          e2 = string_cspn (f2, Ifs, '\n');
+          f2 = string_spn (e2, Ifs, '\n');
 	  /* and then increment the field index */
 	  fieldno2++; 
 	}
@@ -138,8 +141,8 @@ lines_differ (char const *s1, char const *s2, int index1, int index2, argslist*
       if (*f1 != '\n' && *f2 != '\n')
 	{
 	  /* Find the ends of the fields */
-	  for (e1 = (char*)f1; *e1 != '\n' && !strchr (ifs, *e1); e1++);
-	  for (e2 = (char*)f2; *e2 != '\n' && !strchr (Ifs, *e2); e2++);
+          e1 = string_cspn (f1, ifs, '\n');
+          e2 = string_cspn (f2, Ifs, '\n');
 	  /*
 	    Mark the ends of the fields before calling acxnum().
 	    But before doing this, save the original characters placed
@@ -164,8 +167,6 @@ lines_differ (char const *s1, char const *s2, int index1, int index2, argslist*
 	       */
 	      *e1 = ch1;
 	      *e2 = ch2;
-	      f1 = e1;
-	      f2 = e2;
 	    }
 	  else
 	    {
@@ -192,8 +193,6 @@ lines_differ (char const *s1, char const *s2, int index1, int index2, argslist*
 		    {
 		      *e1 = ch1;
 		      *e2 = ch2;
-		      f1 = e1;
-		      f2 = e2;
 		    }
 		}
 	      else
@@ -220,9 +219,9 @@ lines_differ (char const *s1, char const *s2, int index1, int index2, argslist*
 	    Move to the next field
 	    and increase the field number
 	   */
-	  for (; *f1 != '\n' && (strchr (ifs, *f1)); f1++);
+          f1 = string_spn (e1, ifs, '\n');
+          f2 = string_spn (e2, Ifs, '\n');
 	  fieldno1++; 
-	  for (; *f2 != '\n' && (strchr (Ifs, *f2)); f2++);
 	  fieldno2++;
 	} /* End else --> if (*f1 != '\n' && *f2!= '\n') */
     } /* end  while (*f1 != '\n' && *f2 != '\n') */
@@ -233,8 +232,8 @@ lines_differ (char const *s1, char const *s2, int index1, int index2, argslist*
   while ( *f1 != '\n' && fieldno1 < fieldno_upper_limit && (ghostmask[fieldno1 >> 3] & 0x80 >> (fieldno1 & 0x7)) )
     {
       /* First move `f1' to the begin of the next field */
-      for (; *f1 != '\n' && !strchr (ifs, *f1); f1++);
-      for (; *f1 != '\n' && (strchr (ifs, *f1)); f1++);
+      e1 = string_cspn (f1, ifs, '\n');
+      f1 = string_spn (e1, ifs, '\n');
       /* and then increment the field index */
       fieldno1++; 
     }
@@ -249,8 +248,8 @@ lines_differ (char const *s1, char const *s2, int index1, int index2, argslist*
   while ( *f2 != '\n' && fieldno2 < fieldno_upper_limit && (Ghostmask[fieldno2 >> 3] & 0x80 >> (fieldno2 & 0x7)) )
     {
       /* First move `f2' to the begin of the next field */
-      for (; *f2 != '\n' && !strchr (Ifs, *f2); f2++);
-      for (; *f2 != '\n' && (strchr (Ifs, *f2)); f2++);
+      e2 = string_cspn (f2, Ifs, '\n');
+      f2 = string_spn (e2, Ifs, '\n');
       /* and then increment the field index */
       fieldno2++; 
     }
